@@ -13,15 +13,15 @@
 # limitations under the License.
 
 from .. import mesonlib, dependencies, build
-from ..coredata import MesonException
+from ..mesonlib import MesonException
 import os
 
 class WindowsModule:
 
     def detect_compiler(self, compilers):
-        for c in compilers:
-            if c.language == 'c' or c.language == 'cpp':
-                return c
+        for l in ('c', 'cpp'):
+            if l in compilers:
+                return compilers[l]
         raise MesonException('Resource compilation requires a C or C++ compiler.')
 
     def compile_resources(self, state, args, kwargs):
@@ -32,9 +32,14 @@ class WindowsModule:
             res_args = extra_args + ['/nologo', '/fo@OUTPUT@', '@INPUT@']
             suffix = 'res'
         else:
-            rescomp = dependencies.ExternalProgram('windres', silent=True)
+            # Pick-up env var WINDRES if set. This is often used for specifying
+            # an arch-specific windres.
+            rescomp_name = os.environ.get('WINDRES', 'windres')
+            rescomp = dependencies.ExternalProgram(rescomp_name, silent=True)
             res_args = extra_args + ['@INPUT@', '@OUTPUT@']
             suffix = 'o'
+        if not rescomp.found():
+            raise MesonException('Could not find Windows resource compiler %s.' % ' '.join(rescomp.get_command()))
         res_files = mesonlib.stringlistify(args)
         res_kwargs = {'output' : '@BASENAME@.' + suffix,
                       'arguments': res_args}
