@@ -19,12 +19,12 @@ import os, subprocess, shutil, sys, signal
 from io import StringIO
 from ast import literal_eval
 import sys, tempfile
+import mesontest
 from mesonbuild import environment
 from mesonbuild import mesonlib
 from mesonbuild import mlog
 from mesonbuild import mesonmain
-from mesonbuild.mesonlib import stringlistify
-from mesonbuild.scripts import meson_test, meson_benchmark
+from mesonbuild.mesonlib import stringlistify, Popen_safe
 import argparse
 import xml.etree.ElementTree as ET
 import time
@@ -63,9 +63,6 @@ class AutoDeletedDir():
                     raise
                 time.sleep(0.1 * (2**i))
 
-passing_tests = 0
-failing_tests = 0
-skipped_tests = 0
 failing_logs = []
 print_debug = 'MESON_PRINT_TEST_OUTPUT' in os.environ
 do_debug = not {'MESON_PRINT_TEST_OUTPUT', 'TRAVIS', 'APPVEYOR'}.isdisjoint(os.environ)
@@ -93,26 +90,20 @@ unity_flags = []
 backend_flags = None
 compile_commands = None
 test_commands = None
-install_commands = None
+install_commands = []
+clean_commands = []
 
 def setup_commands(backend):
-    global backend_flags, compile_commands, test_commands, install_commands
+    global backend_flags, compile_commands, test_commands, install_commands, clean_commands
     msbuild_exe = shutil.which('msbuild')
-    if backend == 'vs2010' or (backend is None and msbuild_exe is not None):
-        backend_flags = ['--backend=vs2010']
+    if (backend and backend.startswith('vs')) or (backend is None and msbuild_exe is not None):
+        backend_flags = ['--backend=' + backend]
         compile_commands = ['msbuild']
         test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
-        install_commands = []
-    elif backend == 'vs2015':
-        backend_flags = ['--backend=vs2015']
-        compile_commands = ['msbuild']
-        test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
-        install_commands = []
     elif backend == 'xcode' or (backend is None and mesonlib.is_osx()):
         backend_flags = ['--backend=xcode']
         compile_commands = ['xcodebuild']
         test_commands = ['xcodebuild', '-target', 'RUN_TESTS']
-        install_commands = []
     else:
         backend_flags = []
         ninja_command = environment.detect_ninja()
@@ -125,6 +116,7 @@ def setup_commands(backend):
         compile_commands += ['-w', 'dupbuild=err']
         test_commands = [ninja_command, 'test', 'benchmark']
         install_commands = [ninja_command, 'install']
+        clean_commands = [ninja_command, 'clean']
 
 def get_relative_files_list_from_dir(fromdir):
     paths = []
@@ -211,8 +203,8 @@ def run_test_inprocess(testdir):
     old_cwd = os.getcwd()
     os.chdir(testdir)
     try:
-        returncode_test = meson_test.run(['meson-private/meson_test_setup.dat'])
-        returncode_benchmark = meson_benchmark.run(['meson-private/meson_benchmark_setup.dat'])
+        returncode_test = mesontest.run(['--no-rebuild'])
+        returncode_benchmark = mesontest.run(['--no-rebuild', '--benchmark', '--logbase', 'benchmarklog'])
     finally:
         sys.stdout = old_stdout
         sys.stderr = old_stderr
@@ -233,17 +225,18 @@ def parse_test_args(testdir):
         pass
     return args
 
-def run_test(skipped, testdir, extra_args, flags, compile_commands, install_commands, should_fail):
+def run_test(skipped, testdir, extra_args, flags, compile_commands, should_fail):
     if skipped:
         return None
     with AutoDeletedDir(tempfile.mkdtemp(prefix='b ', dir='.')) as build_dir:
         with AutoDeletedDir(tempfile.mkdtemp(prefix='i ', dir=os.getcwd())) as install_dir:
             try:
-                return _run_test(testdir, build_dir, install_dir, extra_args, flags, compile_commands, install_commands, should_fail)
+                return _run_test(testdir, build_dir, install_dir, extra_args, flags, compile_commands, should_fail)
             finally:
                 mlog.shutdown() # Close the log file because otherwise Windows wets itself.
 
-def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_commands, install_commands, should_fail):
+def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_commands, should_fail):
+    global install_commands, clean_commands
     test_args = parse_test_args(testdir)
     gen_start = time.time()
     gen_command = [meson_command, '--prefix', '/usr', '--libdir', 'lib', testdir, test_build_dir]\
@@ -268,12 +261,10 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
     else:
         comp = compile_commands
     build_start = time.time()
-    pc = subprocess.Popen(comp, cwd=test_build_dir,
-                          stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    (o, e) = pc.communicate()
+    pc, o, e = Popen_safe(comp, cwd=test_build_dir)
     build_time = time.time() - build_start
-    stdo += o.decode(sys.stdout.encoding)
-    stde += e.decode(sys.stdout.encoding)
+    stdo += o
+    stde += e
     if should_fail == 'build':
         if pc.returncode != 0:
             return TestResult('', stdo, stde, mesonlog, gen_time)
@@ -294,19 +285,24 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Test that should have failed to run unit tests succeeded', stdo, stde, mesonlog, gen_time)
     if returncode != 0:
         return TestResult('Running unit tests failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
+    # Do installation
     if len(install_commands) == 0:
         return TestResult('', '', '', gen_time, build_time, test_time)
-    else:
+    env = os.environ.copy()
+    env['DESTDIR'] = install_dir
+    pi, o, e = Popen_safe(install_commands, cwd=test_build_dir, env=env)
+    stdo += o
+    stde += e
+    if pi.returncode != 0:
+        return TestResult('Running install failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
+    if len(clean_commands) != 0:
         env = os.environ.copy()
-        env['DESTDIR'] = install_dir
-        pi = subprocess.Popen(install_commands, cwd=test_build_dir, env=env,
-                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        (o, e) = pi.communicate()
-        stdo += o.decode(sys.stdout.encoding)
-        stde += e.decode(sys.stdout.encoding)
+        pi, o, e = Popen_safe(clean_commands, cwd=test_build_dir, env=env)
+        stdo += o
+        stde += e
         if pi.returncode != 0:
-            return TestResult('Running install failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
-        return TestResult(validate_install(testdir, install_dir), stdo, stde, mesonlog, gen_time, build_time, test_time)
+            return TestResult('Running clean failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
+    return TestResult(validate_install(testdir, install_dir), stdo, stde, mesonlog, gen_time, build_time, test_time)
 
 def gather_tests(testdir):
     tests = [t.replace('\\', '/').split('/', 2)[2] for t in glob(os.path.join(testdir, '*'))]
@@ -371,16 +367,27 @@ def detect_tests_to_run():
     all_tests.append(('python3', gather_tests('test cases/python3'), False if using_backend('ninja') and shutil.which('python3') else True))
     return all_tests
 
-def run_tests(extra_args):
-    global passing_tests, failing_tests, stop, executor, futures
-    all_tests = detect_tests_to_run()
-    logfile = open('meson-test-run.txt', 'w', encoding="utf_8")
+def run_tests(all_tests, log_name_base, extra_args):
+    global stop, executor, futures
+    txtname = log_name_base + '.txt'
+    xmlname = log_name_base + '.xml'
+    logfile = open(txtname, 'w', encoding="utf_8")
     junit_root = ET.Element('testsuites')
     conf_time = 0
     build_time = 0
     test_time = 0
+    passing_tests = 0
+    failing_tests = 0
+    skipped_tests = 0
 
-    executor = conc.ProcessPoolExecutor(max_workers=multiprocessing.cpu_count())
+    try:
+        # This fails in some CI environments for unknown reasons.
+        num_workers = multiprocessing.cpu_count()
+    except Exception as e:
+        print('Could not determine number of CPUs due to the following reason:' + str(e))
+        print('Defaulting to using only one process')
+        num_workers = 1
+    executor = conc.ProcessPoolExecutor(max_workers=num_workers)
 
     for name, test_cases, skipped in all_tests:
         current_suite = ET.SubElement(junit_root, 'testsuite', {'name' : name, 'tests' : str(len(test_cases))})
@@ -397,7 +404,7 @@ def run_tests(extra_args):
             should_fail = False
             if name.startswith('failing'):
                 should_fail = name.split('failing-')[1]
-            result = executor.submit(run_test, skipped, t, extra_args, unity_flags + backend_flags, compile_commands, install_commands, should_fail)
+            result = executor.submit(run_test, skipped, t, extra_args, unity_flags + backend_flags, compile_commands, should_fail)
             futures.append((testname, t, result))
         for (testname, t, result) in futures:
             result = result.result()
@@ -406,7 +413,6 @@ def run_tests(extra_args):
                 current_test = ET.SubElement(current_suite, 'testcase', {'name' : testname,
                                                                          'classname' : name})
                 ET.SubElement(current_test, 'skipped', {})
-                global skipped_tests
                 skipped_tests += 1
             else:
                 without_install = "" if len(install_commands) > 0 else " (without install)"
@@ -436,7 +442,8 @@ def run_tests(extra_args):
     print("\nTotal configuration time: %.2fs" % conf_time)
     print("Total build time: %.2fs" % build_time)
     print("Total test time: %.2fs" % test_time)
-    ET.ElementTree(element=junit_root).write('meson-test-run.xml', xml_declaration=True, encoding='UTF-8')
+    ET.ElementTree(element=junit_root).write(xmlname, xml_declaration=True, encoding='UTF-8')
+    return (passing_tests, failing_tests, skipped_tests)
 
 def check_file(fname):
     linenum = 1
@@ -533,7 +540,8 @@ if __name__ == '__main__':
     check_format()
     pbfiles = generate_prebuilt()
     try:
-        run_tests(options.extra_args)
+        all_tests = detect_tests_to_run()
+        (passing_tests, failing_tests, skipped_tests) = run_tests(all_tests, 'meson-test-run', options.extra_args)
     except StopException:
         pass
     for f in pbfiles:
