@@ -5,7 +5,8 @@ import sys
 plain_arg = sys.argv[1]
 _, filename, _ = plain_arg.split(':')
 try:
-    content = open(filename, 'rb').read()
+    with open(filename, 'rb') as f:
+        content = f.read()
 except FileNotFoundError:
     print('Could not open file. Missing dependency?')
     sys.exit(1)
